@@ -68,6 +68,8 @@ void PositionerLayout::addItem(QQuickItem *item)
 
     m_items.insert(item);
 
+    debug("Add item to layout", item);
+
     markDirty();
 }
 
@@ -80,11 +82,23 @@ void PositionerLayout::removeItem(QQuickItem *item)
 
     (*itr)->disconnect(this);
     (*itr)->removeEventFilter(this);
+
+    debug("Remove item from layout", *itr);
     m_items.erase(itr);
 
     markDirty();
 }
 
+bool PositionerLayout::isDebugEnabled() const
+{
+    return m_debugEnabled;
+}
+
+void PositionerLayout::setDebugEnabled(bool newDebug)
+{
+    m_debugEnabled = newDebug;
+}
+
 QSizeF PositionerLayout::implicitSize() const
 {
     return m_implicitSize;
@@ -122,10 +136,13 @@ void PositionerLayout::updatePolish()
     LayoutContainer contentRelative;
     LayoutContainer backgroundRelative;
 
+    debug("Performing layout for positioner of", parentItem());
+
     for (auto &item : m_items) {
         auto source = PositionerSource::Source::Layout;
 
         if (!item->isVisible()) {
+            debug("  Ignoring invisible item", item);
             continue;
         }
 
@@ -142,20 +159,24 @@ void PositionerLayout::updatePolish()
             break;
         }
 
-        std::optional<Union::Properties::AlignmentProperty> alignment;
+        auto properties = query->properties();
+        auto layout = properties->layout();
+
+        Union::Properties::AlignmentProperty *alignment;
         switch (source) {
         case PositionerSource::Source::Layout:
-            alignment = query->properties().layout().value_or(LayoutProperty{}).alignment();
+            alignment = layout ? layout->alignment() : nullptr;
             break;
         case PositionerSource::Source::Icon:
-            alignment = query->properties().icon().value_or(IconProperty{}).alignment();
+            alignment = properties->icon() ? properties->icon()->alignment() : nullptr;
             break;
         case PositionerSource::Source::Text:
-            alignment = query->properties().text().value_or(TextProperty{}).alignment();
+            alignment = properties->text() ? properties->text()->alignment() : nullptr;
             break;
         }
 
         if (!alignment) {
+            debug("  No alignment found for item", item);
             continue;
         }
 
@@ -180,10 +201,9 @@ void PositionerLayout::updatePolish()
             .item = item,
         };
 
-        if (source == PositionerSource::Source::Layout) {
-            auto margins = query->properties().layout().value_or(LayoutProperty{}).margins().value_or(SizeProperty{});
-            layoutItem.margins =
-                QMarginsF(margins.left().value_or(0.0), margins.top().value_or(0.0), margins.right().value_or(0.0), margins.bottom().value_or(0.0));
+        if (source == PositionerSource::Source::Layout && layout && layout->margins()) {
+            auto margins = layout->margins()->toMargins();
+            layoutItem.margins = QMarginsF(margins.left(), margins.top(), margins.right(), margins.bottom());
         }
 
         LayoutContainer *container = &itemRelative;
@@ -223,15 +243,23 @@ void PositionerLayout::updatePolish()
             container->fill.stacked = container->fill.stacked || stacked;
             break;
         }
+
+        debug("  Layout item", item);
+        debug("    Implicit Size:", layoutItem.implicitSize);
+        debug("    Container:", alignment->container());
+        debug("    Alignment: (h)", horizontalAlignment, "(v)", verticalAlignment);
+        debug("    Order:", layoutItem.order);
     }
 
     const auto containerItem = parentItem();
     const auto query = qobject_cast<QuickStyle *>(qmlAttachedPropertiesObject<QuickStyle>(containerItem, true))->query();
-    if (!query) {
+    if (!query || !query->properties()) {
         polish();
         return;
     }
-    qreal spacing = query->properties().layout().value_or(LayoutProperty{}).spacing().value_or(0.0);
+    auto properties = query->properties();
+
+    qreal spacing = properties->layout() ? properties->layout()->spacing().value_or(0.0) : 0.0;
 
     auto sort = [](auto &container) {
         std::stable_sort(container.begin(), container.end(), [](auto first, auto second) {
@@ -258,37 +286,23 @@ void PositionerLayout::updatePolish()
     auto remainingPosition = QPointF{spacedSize(itemRelative.start.size.width(), spacing), 0.0};
     auto remainingSize = itemRelative.size - QSizeF{spacedSize({itemRelative.start.size.width(), itemRelative.end.size.width()}, spacing), 0.0};
 
-    auto inset = query->properties().layout().value_or(LayoutProperty{}).inset().value_or(SizeProperty{}).toMargins();
+    auto inset = properties->layout() && properties->layout()->inset() ? properties->layout()->inset()->toMargins() : QMarginsF{};
     backgroundRelative.position = remainingPosition + QPointF{inset.left(), inset.top()};
     backgroundRelative.size = remainingSize - QSizeF{inset.left() + inset.right(), inset.top() + inset.bottom()};
     layoutContainer(backgroundRelative);
 
-    auto padding = query->properties().layout().value_or(LayoutProperty{}).padding().value_or(SizeProperty{}).toMargins();
+    auto padding = properties->layout() && properties->layout()->padding() ? properties->layout()->padding()->toMargins() : QMarginsF{};
     contentRelative.position = remainingPosition + QPointF{padding.left(), padding.top()};
     contentRelative.size = remainingSize - QSizeF{padding.left() + padding.right(), padding.top() + padding.bottom()};
     layoutContainer(contentRelative);
 
-    for (auto container : {&itemRelative, &backgroundRelative, &contentRelative}) {
-        for (auto bucket : {&container->start, &container->end, &container->center, &container->fill}) {
-            auto position = bucket->position;
-            for (auto item : bucket->items) {
-                auto itemPosition = position + item.position;
-                auto mapped = parentItem()->mapToItem(item.item->parentItem(), itemPosition);
-                item.item->setX(mapped.x());
-                item.item->setY(mapped.y());
-                item.item->setWidth(item.size.width());
-                item.item->setHeight(item.size.height());
-            }
-        }
-    }
-
     auto implicitCenterWidth = std::max(backgroundRelative.implicitSize.width() + inset.left() + inset.right(),
                                         contentRelative.implicitSize.width() + padding.left() + padding.right());
     auto implicitWidth = spacedSize({itemRelative.start.implicitSize.width(), implicitCenterWidth, itemRelative.end.implicitSize.width()}, spacing);
     auto implicitHeight = std::max({itemRelative.implicitSize.height(),
                                     backgroundRelative.implicitSize.height() + inset.top() + inset.bottom(),
                                     contentRelative.implicitSize.height() + padding.top() + padding.bottom()});
-    auto newImplicitSize = QSizeF{std::round(implicitWidth), std::round(implicitHeight)};
+    auto newImplicitSize = QSizeF{std::ceil(implicitWidth), std::ceil(implicitHeight)};
     if (newImplicitSize != m_implicitSize) {
         m_implicitSize = newImplicitSize;
         Q_EMIT implicitSizeChanged();
@@ -300,10 +314,27 @@ void PositionerLayout::updatePolish()
                             contentRelative.position.y(),
                             bottomRight.height() - contentRelative.position.y());
     if (m_padding != newPadding) {
+        debug("  Updating padding to", newPadding.toMargins());
         m_padding = newPadding;
         Q_EMIT paddingChanged();
     }
 
+    for (auto container : {&itemRelative, &backgroundRelative, &contentRelative}) {
+        for (auto bucket : {&container->start, &container->end, &container->center, &container->fill}) {
+            auto position = bucket->position;
+            for (auto item : bucket->items) {
+                auto itemPosition = position + item.position;
+                auto mapped = parentItem()->mapToItem(item.item->parentItem(), itemPosition);
+                item.item->setX(mapped.x());
+                item.item->setY(mapped.y());
+                item.item->setWidth(item.size.width());
+                item.item->setHeight(item.size.height());
+
+                debug("  Item", item.item, "positioned at", itemPosition, mapped);
+            }
+        }
+    }
+
     m_layouting = false;
 
     Q_EMIT layoutFinished();
@@ -361,17 +392,17 @@ void PositionerLayout::layoutContainer(LayoutContainer &container)
             switch (item.verticalAlignment) {
             case Union::Properties::Alignment::Unspecified:
             case Union::Properties::Alignment::Start:
-                item.position.setY(0.0);
+                item.position.setY(item.margins.top());
                 break;
             case Union::Properties::Alignment::Center:
-                item.position.setY(std::round(bucket->size.height() / 2 - item.implicitSize.height() / 2));
+                item.position.setY(std::round(bucket->size.height() / 2 - (item.implicitSize.height() + item.margins.top() + item.margins.bottom()) / 2));
                 break;
             case Union::Properties::Alignment::End:
-                item.position.setY(bucket->size.height() - item.implicitSize.height());
+                item.position.setY(bucket->size.height() - item.implicitSize.height() - item.margins.bottom());
                 break;
             case Union::Properties::Alignment::Fill:
-                item.position.setY(0);
-                item.size.setHeight(bucket->size.height());
+                item.position.setY(item.margins.top());
+                item.size.setHeight(bucket->size.height() - item.margins.top() - item.margins.bottom());
                 break;
             case Union::Properties::Alignment::Stack:
                 stackedY += item.margins.top();
@@ -397,17 +428,25 @@ void PositionerLayout::layoutBucket(LayoutBucket &bucket)
             item.position.setX(0.0);
             totalHeight += item.margins.top() + item.margins.bottom() + item.implicitSize.height();
         } else {
+            if (qFuzzyIsNull(item.implicitSize.width())) {
+                continue;
+            }
+
+            if (x > 0.0) {
+                x += bucket.spacing;
+            }
+
             x += item.margins.left();
             item.position.setX(x);
-            x += item.implicitSize.width() + item.margins.right() + bucket.spacing;
+            x += item.implicitSize.width() + item.margins.right();
         }
 
         maxWidth = std::max(maxWidth, item.margins.left() + item.implicitSize.width() + item.margins.right());
         maxHeight = std::max(maxHeight, item.margins.top() + item.implicitSize.height() + item.margins.bottom());
     }
 
-    bucket.implicitSize.setWidth(std::round(std::max(bucket.stacked ? maxWidth : x - bucket.spacing, 0.0)));
-    bucket.implicitSize.setHeight(std::round(std::max(bucket.stacked ? totalHeight : maxHeight, 0.0)));
+    bucket.implicitSize.setWidth(std::ceil(std::max(bucket.stacked ? maxWidth : x, 0.0)));
+    bucket.implicitSize.setHeight(std::ceil(std::max(bucket.stacked ? totalHeight : maxHeight, 0.0)));
 }
 
 void PositionerLayout::onParentSizeChanged()
