@@ -42,6 +42,11 @@ bool StyledRectangle::event(QEvent *event)
         return false;
     }
 
+    if (event->type() == QuickStyleColorsChangedEvent::s_type) {
+        update();
+        return false;
+    }
+
     return QQuickItem::event(event);
 }
 
@@ -64,7 +69,7 @@ QSGNode *StyledRectangle::updatePaintNode(QSGNode *node, QQuickItem::UpdatePaint
         return nullptr;
     }
 
-    if (!query->properties().hasAnyValue()) {
+    if (!query->properties()->hasAnyValue()) {
         return nullptr;
     }
 
@@ -73,23 +78,10 @@ QSGNode *StyledRectangle::updatePaintNode(QSGNode *node, QQuickItem::UpdatePaint
         return nullptr;
     }
 
-    auto style = query->properties();
-    auto minSize = minimumSize(style);
-
-    // If we cannot fit the background image into the required rect, just don't
-    // display anything.'
-    if (rect.width() < minSize.width() || rect.height() < minSize.height()) {
-        return nullptr;
-    }
-
     // Ensure we *don't* delete the node at this point.
     guard.dismiss();
 
-    if (style.border_or_new().left_or_new().image_or_new().hasAnyValue()) {
-        return updateRectangleNode(node, style);
-    } else {
-        return updateShaderNode(node, style);
-    }
+    return updateShaderNode(node, query->properties());
 }
 
 void StyledRectangle::updateImplicitSize()
@@ -103,62 +95,15 @@ void StyledRectangle::updateImplicitSize()
         return;
     }
 
-    if (query->properties().layout().has_value()) {
-        auto layout = query->properties().layout().value();
-        setImplicitSize(layout.width().value_or(0.0), layout.height().value_or(0.0));
+    if (query->properties()->layout()) {
+        auto layout = query->properties()->layout();
+        setImplicitSize(layout->width().value_or(0.0), layout->height().value_or(0.0));
     }
 
     update();
 }
 
-QSizeF StyledRectangle::minimumSize(const Properties::StyleProperty &style)
-{
-    if (!style.corners() && !style.border()) {
-        return QSizeF{0.0, 0.0};
-    }
-
-    auto corners = style.corners_or_new();
-    auto border = style.border_or_new();
-
-    auto left = std::initializer_list<qreal>{
-        corners.topLeft_or_new().width().value_or(0.0),
-        border.left_or_new().size().value_or(0.0),
-        corners.bottomLeft_or_new().width().value_or(0.0),
-    };
-    auto right = std::initializer_list<qreal>{
-        corners.topRight_or_new().width().value_or(0.0),
-        border.right_or_new().size().value_or(0.0),
-        corners.bottomRight_or_new().width().value_or(0.0),
-    };
-    auto top = std::initializer_list<qreal>{
-        corners.topLeft_or_new().height().value_or(0.0),
-        border.top_or_new().size().value_or(0.0),
-        corners.topRight_or_new().height().value_or(0.0),
-    };
-    auto bottom = std::initializer_list<qreal>{
-        corners.bottomLeft_or_new().height().value_or(0.0),
-        border.bottom_or_new().size().value_or(0.0),
-        corners.bottomRight_or_new().height().value_or(0.0),
-    };
-
-    return QSizeF{std::max(left) + std::max(right), std::max(top) + std::max(bottom)};
-}
-
-QSGNode *StyledRectangle::updateRectangleNode(QSGNode *node, const Union::Properties::StyleProperty &style)
-{
-    if (!node) {
-        node = new RectangleNode{};
-    }
-
-    auto rectangleNode = static_cast<RectangleNode *>(node);
-    rectangleNode->setStyle(style);
-    rectangleNode->rect = boundingRect();
-    rectangleNode->update(window());
-
-    return rectangleNode;
-}
-
-QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty &style)
+QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty *style)
 {
     if (!node) {
         node = new QSGNode{};
@@ -166,11 +111,10 @@ QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty &s
 
     auto rect = boundingRect();
 
-    auto corners = style.corners_or_new();
-    auto radii = QVector4D{float(corners.topRight_or_new().radius().value_or(0.0)),
-                           float(corners.bottomRight_or_new().radius().value_or(0.0)),
-                           float(corners.topLeft_or_new().radius().value_or(0.0)),
-                           float(corners.bottomLeft_or_new().radius().value_or(0.0))};
+    auto cornerSizes = style->corners() ? style->corners()->radii() : CornersProperty::CornerRadii{};
+
+    // Shader corner radius order is bottom right, top right, bottom left, top left.
+    auto radii = QVector4D{float(cornerSizes.bottomRight), float(cornerSizes.topRight), float(cornerSizes.bottomLeft), float(cornerSizes.topLeft)};
 
     OutlineBorderRectangleNode *borderNode = nullptr;
 
@@ -178,7 +122,7 @@ QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty &s
     // This is done because the shadow geometry can vary greatly from the
     // rectangle geometry and separating them into different nodes made the
     // whole thing a lot simpler.
-    if (auto shadow = style.shadow(); shadow.has_value() && !shadow->isEmpty()) {
+    if (auto shadow = style->shadow(); shadow && !shadow->isEmpty()) {
         if (node->childCount() == 0) {
             node->appendChildNode(new RectangleShadowNode{});
             node->appendChildNode(new OutlineBorderRectangleNode{});
@@ -191,7 +135,7 @@ QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty &s
         shadowNode->setRadius(radii);
         shadowNode->setBlur(shadow->blur().value_or(0.0));
         shadowNode->setSpread(shadow->size().value_or(0.0));
-        shadowNode->setOffset(shadow->offset_or_new().toVector2D());
+        shadowNode->setOffset(shadow->offset() ? shadow->offset()->toVector2D() : QVector2D{});
         shadowNode->setColor(shadow->color().value_or(Union::Color{}).toQColor());
         shadowNode->update();
 
@@ -209,9 +153,9 @@ QSGNode *StyledRectangle::updateShaderNode(QSGNode *node, const StyleProperty &s
     }
 
     borderNode->m_itemRect = rect;
-    borderNode->m_background = style.background_or_new();
-    borderNode->m_border = style.border();
-    borderNode->m_outline = style.outline();
+    borderNode->m_background = style->background();
+    borderNode->m_border = style->border();
+    borderNode->m_outline = style->outline();
     borderNode->m_radius = radii;
     borderNode->m_window = window();
     borderNode->update();
