@@ -4,7 +4,9 @@
 #include "KColorSchemeProvider.h"
 
 #include <KColorUtils>
+#include <QCoreApplication>
 #include <QMetaEnum>
+#include <Style.h>
 
 #include "kcolorscheme_logging.h"
 
@@ -102,6 +104,20 @@ std::optional<KColorScheme::DecorationRole> decorationRoleFromString(const QStri
 KColorSchemeProvider::KColorSchemeProvider(QObject *parent)
     : ColorProvider(parent)
 {
+    qApp->installEventFilter(this);
+
+    m_colorConfig = KSharedConfig::openConfig();
+}
+
+bool KColorSchemeProvider::eventFilter(QObject *obj, QEvent *event)
+{
+    if (obj == qApp && event->type() == QEvent::ApplicationPaletteChange) {
+        m_cache.clear();
+        // Important: If we don't reparse the global configuration, we will not
+        // get any new colors.
+        m_colorConfig->reparseConfiguration();
+    }
+    return QObject::eventFilter(obj, event);
 }
 
 std::optional<Union::ColorProvider::Rgba> KColorSchemeProvider::color(const QStringList &arguments) const
@@ -128,7 +144,7 @@ std::optional<Union::ColorProvider::Rgba> KColorSchemeProvider::color(const QStr
         return std::nullopt;
     }
 
-    KColorScheme colorScheme(group.value(), colorSet.value());
+    KColorScheme colorScheme(group.value(), colorSet.value(), m_colorConfig);
 
     auto rgbaFromQColor = [](const QColor &qcolor) {
         return Rgba{
@@ -159,7 +175,7 @@ std::optional<Union::ColorProvider::Rgba> KColorSchemeProvider::color(const QStr
         if (arguments.at(3) == u"frame") {
             QColor framecolor = KColorUtils::mix(colorScheme.background(KColorScheme::BackgroundRole::NormalBackground).color(),
                                                  colorScheme.foreground(KColorScheme::ForegroundRole::NormalText).color(),
-                                                 KColorScheme::frameContrast());
+                                                 KColorScheme::frameContrast(m_colorConfig));
             value = rgbaFromQColor(framecolor);
         } else {
             auto role = decorationRoleFromString(arguments.at(3));
